@@ -57,6 +57,8 @@ typedef struct _WnckTaskClass   WnckTaskClass;
 #define DEFAULT_WIDTH 1
 #define DEFAULT_HEIGHT 48
 
+#define TIMEOUT_ACTIVATE 1000
+
 struct _WnckTask
 {
   GObject parent_instance;
@@ -67,7 +69,6 @@ struct _WnckTask
   GtkWidget *image;
   GtkWidget *label;
   
-  gboolean is_application;
   WnckApplication *application;
   WnckWindow *window;
 
@@ -75,8 +76,6 @@ struct _WnckTask
 
   GList *windows; /* List of the WnckTask for the window,
 		     if this is an application */
-  gboolean really_toggling; /* Set when tasklist really wants
-			       to change the togglebutton state */
   gulong state_changed_tag;
   gulong icon_changed_tag;
   gulong name_changed_tag;
@@ -86,6 +85,14 @@ struct _WnckTask
   GtkWidget *menu;
   /* ops menu */
   GtkWidget *action_menu;
+
+  guint is_application : 1;  /* TRUE if this task is a grouped application */
+  guint really_toggling : 1; /* Set when tasklist really wants
+                              * to change the togglebutton state
+                              */
+  guint was_active : 1;      /* used to fixup activation behavior */ 
+
+  guint button_activate;
 };
 
 struct _WnckTaskClass
@@ -125,6 +132,7 @@ struct _WnckTasklistPrivate
   int size_hints_len;
 
   gint minimum_width;
+  gint minimum_height;
 };
 
 
@@ -171,9 +179,14 @@ static void     wnck_tasklist_active_window_changed    (WnckScreen   *screen,
 							WnckTasklist *tasklist);
 static void     wnck_tasklist_active_workspace_changed (WnckScreen   *screen,
 							WnckTasklist *tasklist);
-static void     wnck_tasklist_window_added_or_removed  (WnckScreen   *screen,
+static void     wnck_tasklist_window_added             (WnckScreen   *screen,
 							WnckWindow   *win,
 							WnckTasklist *tasklist);
+static void     wnck_tasklist_window_removed           (WnckScreen   *screen,
+							WnckWindow   *win,
+							WnckTasklist *tasklist);
+static void     wnck_tasklist_connect_window           (WnckTasklist *tasklist,
+							WnckWindow   *window);
 
 static void     wnck_tasklist_change_active_task       (WnckTasklist *tasklist,
 							WnckTask *active_task);
@@ -314,7 +327,12 @@ wnck_task_finalize (GObject *object)
       task->application = NULL;
     }
 
-  
+  if (task->button_activate != 0)
+    {
+      g_source_remove (task->button_activate);
+      task->button_activate = 0;
+    } 
+
   G_OBJECT_CLASS (task_parent_class)->finalize (object);
 }
 
@@ -364,6 +382,7 @@ wnck_tasklist_init (WnckTasklist *tasklist)
   tasklist->priv->grouping_limit = DEFAULT_GROUPING_LIMIT;
 
   tasklist->priv->minimum_width = DEFAULT_WIDTH;
+  tasklist->priv->minimum_height = DEFAULT_HEIGHT;
 }
 
 static void
@@ -474,12 +493,15 @@ wnck_tasklist_set_grouping_limit (WnckTasklist *tasklist,
   gtk_widget_queue_resize (GTK_WIDGET (tasklist));
 }
 
-/* set the minimum width */
+/* set the minimum width 
+ * use -1 to unset it (resulting in the default width */
 void 
 wnck_tasklist_set_minimum_width (WnckTasklist *tasklist, gint size)
 {
   g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
 
+  if (size == -1) size = DEFAULT_WIDTH;
+
   if (tasklist->priv->minimum_width == size)
     return;
 
@@ -496,6 +518,31 @@ wnck_tasklist_get_minimum_width (WnckTasklist *tasklist)
   return tasklist->priv->minimum_width;
 }
 
+/* set the minimum height
+ * use -1 to unset it (resulting in the default height */
+void 
+wnck_tasklist_set_minimum_height (WnckTasklist *tasklist, gint size)
+{
+  g_return_if_fail (WNCK_IS_TASKLIST (tasklist));
+
+  if (size == -1) size = DEFAULT_HEIGHT;
+
+  if (tasklist->priv->minimum_height == size)
+    return;
+
+  tasklist->priv->minimum_height = size;
+  gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+}
+ 
+/* get the minimum height */
+gint
+wnck_tasklist_get_minimum_height (WnckTasklist *tasklist)
+{
+  g_return_val_if_fail (WNCK_IS_TASKLIST (tasklist), 0);
+	
+  return tasklist->priv->minimum_height;
+}
+
 /* returns the maximal possible button width (i.e. if you
  * don't want to stretch the buttons to fill the alloctions
  * the width can be smaller) */
@@ -618,7 +665,7 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   GtkAllocation  fake_allocation;
   int max_height = 1;
   int max_width = 1;
-  int u_width, u_height;
+  /* int u_width, u_height; */
   GList *l;
   GArray *array;
   GList *ungrouped_apps;
@@ -669,11 +716,13 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
   tasklist->priv->max_button_height = max_height;
 
 
+  /* this snippet of code seems to want to allocate at least
+   * four buttons' worth of height for the widget.  I think this isn't
+   * necessary any more now that we have the minimum_size option, so
+   * I'm commenting this out - thomasvs */
+  /*
   gtk_widget_get_size_request (widget, &u_width, &u_height);
 
-  requisition->width = tasklist->priv->minimum_width;
-  requisition->height = DEFAULT_HEIGHT;
-  
   if (u_height != -1)
     {
       requisition->height = u_height;
@@ -684,6 +733,12 @@ wnck_tasklist_size_request  (GtkWidget      *widget,
       requisition->height = 4 * max_height;
     }
   
+  requisition->width = MAX(requisition->width, tasklist->priv->minimum_width);
+  requisition->height = MAX(requisition->height, tasklist->priv->minimum_height);
+  */
+  requisition->width = tasklist->priv->minimum_width;
+  requisition->height = tasklist->priv->minimum_height;
+  
   fake_allocation.width = requisition->width;
   fake_allocation.height = requisition->height;
 
@@ -1001,6 +1056,7 @@ GtkWidget*
 wnck_tasklist_new (WnckScreen *screen)
 {
   WnckTasklist *tasklist;
+  GList        *windows;
 
   tasklist = g_object_new (WNCK_TYPE_TASKLIST, NULL);
 
@@ -1019,12 +1075,19 @@ wnck_tasklist_new (WnckScreen *screen)
 			   G_CALLBACK (wnck_tasklist_active_workspace_changed),
 			   tasklist, 0);
   g_signal_connect_object (G_OBJECT (screen), "window_opened",
-			   G_CALLBACK (wnck_tasklist_window_added_or_removed),
+			   G_CALLBACK (wnck_tasklist_window_added),
 			   tasklist, 0);
   g_signal_connect_object (G_OBJECT (screen), "window_closed",
-			   G_CALLBACK (wnck_tasklist_window_added_or_removed),
+			   G_CALLBACK (wnck_tasklist_window_removed),
 			   tasklist, 0);
-  
+
+  windows = wnck_screen_get_windows (screen);
+  while (windows != NULL)
+    {
+      wnck_tasklist_connect_window (tasklist, windows->data);
+      windows = windows->next;
+    }
+
   return GTK_WIDGET (tasklist);
 }
 
@@ -1246,9 +1309,67 @@ wnck_tasklist_active_workspace_changed (WnckScreen   *screen,
 }
 
 static void
-wnck_tasklist_window_added_or_removed (WnckScreen   *screen,
-				       WnckWindow   *win,
-				       WnckTasklist *tasklist)
+wnck_tasklist_window_changed_workspace (WnckWindow   *window,
+					WnckTasklist *tasklist)
+{
+  WnckWorkspace *active_ws;
+  WnckWorkspace *window_ws;
+  gboolean       need_update;
+  GList         *l;
+
+  active_ws = wnck_screen_get_active_workspace (tasklist->priv->screen);
+  window_ws = wnck_window_get_workspace (window);
+
+  if (!window_ws)
+    return;
+
+  need_update = FALSE;
+
+  if (active_ws == window_ws)
+    need_update = TRUE;
+
+  l = tasklist->priv->windows;
+  while (!need_update && l != NULL)
+    {
+      WnckTask *task = l->data;
+
+      if (!task->is_application && task->window == window)
+        need_update = TRUE;
+
+      l = l->next;
+    }
+
+  if (need_update)
+    {
+      wnck_tasklist_update_lists (tasklist);
+      gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+    }
+}
+
+static void
+wnck_tasklist_connect_window (WnckTasklist *tasklist,
+			      WnckWindow   *window)
+{
+  g_signal_connect_object (window, "workspace_changed",
+			   G_CALLBACK (wnck_tasklist_window_changed_workspace),
+			   tasklist, 0);
+}
+
+static void
+wnck_tasklist_window_added (WnckScreen   *screen,
+			    WnckWindow   *win,
+			    WnckTasklist *tasklist)
+{
+  wnck_tasklist_connect_window (tasklist, win);
+
+  wnck_tasklist_update_lists (tasklist);
+  gtk_widget_queue_resize (GTK_WIDGET (tasklist));
+}
+
+static void
+wnck_tasklist_window_removed (WnckScreen   *screen,
+			      WnckWindow   *win,
+			      WnckTasklist *tasklist)
 {
   wnck_tasklist_update_lists (tasklist);
   gtk_widget_queue_resize (GTK_WIDGET (tasklist));
@@ -1333,8 +1454,9 @@ wnck_tasklist_activate_task_window     (WnckTask *task)
     }
   else
     {
-      if (wnck_window_is_active (task->window))
+      if (task->was_active)
 	{
+	  task->was_active = FALSE;
 	  wnck_window_minimize (task->window);
 	  return;
 	}
@@ -1679,6 +1801,49 @@ wnck_task_app_name_changed (WnckApplication *app,
     wnck_task_update_visible_state (task);
 }
 
+static gboolean
+wnck_task_motion_timeout (gpointer data)
+{
+  WnckTask *task = WNCK_TASK (data);
+
+  task->button_activate = 0;
+
+  wnck_window_activate (task->window);
+
+  return FALSE;
+}
+
+static void
+wnck_task_drag_leave (GtkWidget          *widget,
+		      GdkDragContext     *context,
+		      guint               time,
+		      WnckTask           *task)
+{
+  if (task->button_activate != 0)
+    {
+      g_source_remove (task->button_activate);
+      task->button_activate = 0;
+    }
+}
+
+static gboolean
+wnck_task_drag_motion (GtkWidget          *widget,
+		       GdkDragContext     *context,
+		       gint                x,
+		       gint                y,
+		       guint               time,
+		       WnckTask            *task)
+{
+
+   if (task->button_activate == 0 && !(task->is_application))
+       task->button_activate = g_timeout_add (TIMEOUT_ACTIVATE,
+                                              wnck_task_motion_timeout,
+                                              task);
+   gdk_drag_status (context,0,time);
+
+   return TRUE;
+}
+
 static gboolean
 wnck_task_button_press_event (GtkWidget	      *widget,
 			      GdkEventButton  *event,
@@ -1692,6 +1857,15 @@ wnck_task_button_press_event (GtkWidget	      *widget,
                             event->button == 3);
       return TRUE;
     }
+  else if (event->button == 1)
+    {
+      if (wnck_window_is_active (task->window))
+        task->was_active = TRUE;
+      else
+        task->was_active = FALSE;
+
+      return FALSE;
+    } 
   else if (event->button == 3)
     {
       if (task->action_menu)
@@ -1715,7 +1889,7 @@ wnck_task_button_press_event (GtkWidget	      *widget,
       
       return TRUE;
     }
-  
+
   return FALSE;
 }
 
@@ -1726,14 +1900,23 @@ wnck_task_create_widgets (WnckTask *task)
   GtkWidget *table;
   GdkPixbuf *pixbuf;
   char *text;
+  static GQuark disable_sound_quark = 0;
   
+  if (!disable_sound_quark)
+    disable_sound_quark = g_quark_from_static_string ("gnome_disable_sound_events");
+
   task->button = gtk_toggle_button_new ();
+  task->button_activate = 0;
+  g_object_set_qdata (G_OBJECT (task->button),
+		      disable_sound_quark, GINT_TO_POINTER (TRUE));
   g_object_add_weak_pointer (G_OBJECT (task->button),
                              (void**) &task->button);
   
   gtk_widget_set_name (task->button,
 		       "tasklist-button");
 
+  gtk_drag_dest_set (GTK_WIDGET(task->button), 0, NULL, 0, 0);
+
   table = gtk_table_new (1, 2, FALSE);
 
   pixbuf = wnck_task_get_icon (task);
@@ -1782,6 +1965,16 @@ wnck_task_create_widgets (WnckTask *task)
                            G_OBJECT (task),
                            0);
 
+  g_signal_connect_object (G_OBJECT(task->button), "drag_motion",
+                           G_CALLBACK (wnck_task_drag_motion),
+                           G_OBJECT (task),
+                           0); 
+
+  g_signal_connect_object (G_OBJECT(task->button), "drag_leave",
+                           G_CALLBACK (wnck_task_drag_leave),
+                           G_OBJECT (task),
+                           0);
+
   if (task->is_application)
     {
       task->app_name_changed_tag = g_signal_connect (G_OBJECT (task->application), "name_changed",
